'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useAction = undefined;

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _parseFunction = require('parse-function');

var _parseFunction2 = _interopRequireDefault(_parseFunction);

var _path = require('path');

var _changeCase = require('@rocketstation/change-case');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const useAction = exports.useAction = (parser, validator, router, routes, route, befores, name, action) => {
  const {
    logging = {},
    method,
    onError,
    schema,
    type = 'post',
    url = `${route}${name}`
  } = action;

  const middlewares = [];

  const onValidationError = error => error.isValidation ? error.errors.map(({ dataPath: key, message: value }) => ({ key, value })) : null;

  middlewares.push(async (ctx, next) => {
    try {
      await next();
    } catch (error) {
      let errors;

      if (error.constructor === Array) errors = error;
      if (!errors && onError) errors = await onError(error);
      if (!errors) errors = onValidationError(error);
      if (!errors) throw error;else return _bluebird2.default.reject({ status: 400, errors });
    }
  }, ...befores);

  if (method) {
    if (schema) {
      const isValid = validator.compile(parser.parse(schema).schema);

      middlewares.push(async (ctx, next) => {
        ctx.request.body = ctx.request.body || {};
        if (logging.body) console.log(JSON.stringify(ctx.request.body));
        if (!isValid(ctx.request.body)) {
          return _bluebird2.default.reject({ errors: isValid.errors, isValidation: true });
        }
        await next();
      });
    }
    router.use(url, ...middlewares);
    router[type](url, async ctx => {
      if (logging.schema) console.log(JSON.stringify(ctx.request.body));
      await method(ctx);
      if (logging.body) console.log(JSON.stringify(ctx.body));
    });
    routes[`${type} ${url}`] = (0, _changeCase.cu)(type, url);
  }
};

const isFile = file => file.includes('.js');

const loadRecursively = (dir, controllers, addService, container, namespaces = []) => {
  if (_fs2.default.existsSync(dir) && _fs2.default.lstatSync(dir).isDirectory()) {
    _fs2.default.readdirSync(dir).filter(item => !item.includes('.') || isFile(item)).forEach(item => {
      if (isFile(item)) {
        const [name] = item.split('.');
        const title = (0, _changeCase.cu)(...[].concat(namespaces, name, 'Controller'));
        const definition = require((0, _path.join)(dir, item));
        const dependencies = (0, _parseFunction2.default)(definition).args;
        addService(title, definition, ...dependencies);
        controllers[name] = container[title];
      } else {
        controllers[item] = { isNamespace: true };
        loadRecursively((0, _path.join)(dir, item), controllers[item], addService, container, namespaces.concat(item));
      }
    });
  }
};

const load = (dir, addService, container) => {
  const controllersDir = (0, _path.join)(dir, 'controllers');
  const controllers = {};

  loadRecursively(controllersDir, controllers, addService, container);

  return _bluebird2.default.resolve(controllers);
};

exports.default = load;